function romInfo = est_conv_mtraj_v2(a_Mtraj,dt, romInfo,File_estimator,saveON)
% Estimate parameter in ROM: Atilde, Btilde, and test their convergence as data size increases 
%      
% Last updated: FL 2022-3-20;  
%{
1. Get a(t) data; --- Load u traj, PODs
2. Estimate Atilde, Btilde for a sequence of numbers of trajs
3. Test convergence 
%}

[r,tN,M]  = size(a_Mtraj);
delta_a   = a_Mtraj(:,2:end,:)-a_Mtraj(:,1:end-1,:);
SigmaT_M  = Sigma_estimate_mtraj_v2(delta_a,dt);    % estimate the covariance by quadratic variation

if ~exist('saveON','var'); saveON =1; end
if ~exist(File_estimator,'file')
    %% 1. regression matrix/vetor ( Abar, bbar) for each trajectory, with Estimator
    SigmaT_all = zeros(r,r,M);
    A_est_all  = zeros(r,r,M);   B_est_all  = zeros(r,r,r,M);  res_all  = zeros(r,M);
    lse_all    = cell(M,1);
    %parfor k_test = start_test:divider:num_test
    for m = 1:M
        da_1traj = delta_a(:,:,m);
        a_1traj  = a_Mtraj(:,:,m);
        sigma1   = Sigma_estimate_mtraj_v2(da_1traj,dt);                                        % estimate the covariance by quadratic variation
        [A_est,B_est,res,lseInfo,romInfo] = parEst_2s_rom_1traj(da_1traj,dt,a_1traj,romInfo); % estimate A and B by least squares, not weighted by SigmaT_M yet
        SigmaT_all(:,:,m) = sigma1;
        A_est_all(:,:,m)  = A_est; B_est_all(:,:,:,m) = B_est;  res_all(:,m) = res';
        lse_all{m}        = lseInfo;
    end
    
    %% 2. Get estimator for a sequence of numbers of trajectories
    seq_size   = 10;                        % length of sequence of estimators for conv test
    [A_est_Mseq,B_est_Mseq,res_seq,M_seq,c1seq] = Estimator_seq(lse_all,romInfo,seq_size);
    
    romInfo.A_est = A_est_Mseq(:,:,end);
    romInfo.B_est = B_est_Mseq(:,:,:,end);
    romInfo.c_est = c1seq(:,end);
    romInfo.res   = res_seq(:,end); 
    if saveON == 1     % save all trajectory-wise estimators 
        save(File_estimator,'A_est_all','B_est_all','res_all','A_est_Mseq','B_est_Mseq','res_seq','M_seq','SigmaT_M','romInfo','c1seq');
    elseif saveON ==2   % save only the M-trajectory estimators in Mseq  
        save(File_estimator,'A_est_Mseq','B_est_Mseq','res_seq','M_seq','SigmaT_M','romInfo','c1seq');
    end
end
if saveON == 1
    load(File_estimator);
    if ~exist('romInfo','var') || ~isfield('romInfo','res')
        romInfo.A_est = A_est_Mseq(:,:,end);
        romInfo.B_est = B_est_Mseq(:,:,:,end);
        romInfo.c_est = c1seq(:,end);
        romInfo.res   = res_seq(:,end);
    end
end

end






