function pod_conv_plot_v2(pod_cov,sig_cov,k_test,pod_modes)
% plot the convergence of the pod basis and the eigenvalues 

Ntest = length(k_test);

err_pods = zeros(Ntest,size(pod_cov{1,1},2));
err_eig = zeros(Ntest,size(pod_cov{1,1},2));

pod_end = pod_cov{end,1};
eig_end = sig_cov{end,1};

for k = 1: Ntest

    diff_pod1 = pod_cov{k,1} - pod_end;
    diff_pod2 = -pod_cov{k,1} - pod_end;

    err1 = diag(sqrt(diff_pod1'*diff_pod1));
    err2 = diag(sqrt(diff_pod2'*diff_pod2));

    err_pods(k,:) = min([err1,err2]');
    
    err_eig(k,:) = abs(sig_cov{k,1}(pod_modes) - eig_end(pod_modes));
end

figure('Renderer', 'painters', 'Position', [10 10 1500 600])
%%
subplot(1,2,1)
loglog(k_test,err_pods(:,1),'LineWidth',2,'Color','r')
hold on
loglog(k_test,err_pods(:,2),'LineWidth',2,'Color','b')
hold on
loglog(k_test,err_pods(:,3),'LineWidth',2,'Color','g')
hold on
loglog(k_test,err_pods(:,4),'LineWidth',2,'Color','k')
hold on
ax = gca;
ax.FontSize = 18;
lgd = legend(['$\phi_{',num2str(pod_modes(1)),'}$'],['$\phi_{',num2str(pod_modes(2)),'}$'],['$\phi_{',num2str(pod_modes(3)),'}$'],['$\phi_{',num2str(pod_modes(4)),'}$']); 
lgd.Interpreter = 'latex';
xlabel('Number of Trajectories','FontSize',20)
ylabel('Errors','FontSize',20)

%xticks(k_test)
title('POD')
%%
subplot(1,2,2)

loglog(k_test,err_eig(:,1),'LineWidth',2,'Color','r')
hold on
loglog(k_test,err_eig(:,2),'LineWidth',2,'Color','b')
hold on
loglog(k_test,err_eig(:,3),'LineWidth',2,'Color','g')
hold on
loglog(k_test,err_eig(:,4),'LineWidth',2,'Color','k')
hold on
ax = gca;
ax.FontSize = 18;
lgd = legend(['$\lambda_{',num2str(pod_modes(1)),'}$'],['$\lambda_{',num2str(pod_modes(2)),'}$'],['$\lambda_{',num2str(pod_modes(3)),'}$'],['$\lambda_{',num2str(pod_modes(4)),'}$']); 
lgd.Interpreter = 'latex';
xlabel('Number of Trajectories','FontSize',20)
ylabel('Errors','FontSize',20)

%xticks(k_test)

title('Eigenvalue')
%title('POD Errors','Interpreter','latex','FontSize',30)
%exportgraphics(gcf,'imagesc_lhs_mat.eps','Resolution',600)

return