function [opt,domain,filter,fem_mesh,fem_settings]= burgers1D_settings() 

%========= user-defined paremeters
%--- specify test problem
opt.es     = 0;          % choose test problem %%just for numbering of the case
opt.rhs    = 0;          % choose equations & rhs
opt.bc     = 0;          % choose boundary conditions
opt.ic     = 2;          % choose initial conditions
opt.theta  = 1;          % choose theta value, if negative, semi-implicit


domain.interval = [0, 1];% spatial interval
domain.elem_order = 2;          % choose the order of element
domain.h   = 1/2^7;      % choose mesh size on spatial space
domain.range = [0, 2];   % time range
domain.t   = 5e-3;       % choose time step on tempral space
domain.tsave = domain.t;
domain.calerr_skip = 1;  % skip some steps when calculate errors

opt.parm_e = 2e-3; % viscosity
%------- filtering option
filter.op =0;
filter.delta =0;%1e-4;


%--- set numerical discretization
n_gauss    = 5;              % Gaussian quadrature in the FE integration

%--- generate 1D mesh
[x,e_conn,bn_index,fem_mesh] = fem1d_mesh(domain, domain.elem_order);     % generate mesh


% x: X-coordinates; e_conn: element info
n_nodes     = size(x,1);
n_elements  = size(e_conn,1);
%--- specify test problem
[u,f,ux,B,Nonlfunc]  = burgers1d_fem_setting(opt,domain);
% u.func(func):  either u or unknown;    u.var(sym): symbolic variable of u.
% f.func(func):  either f or unknown;    f.var(sym): symbolic variable of f.
% ux.func(func): either du/dx or unknow; ux.var(sym):symbolic variable of ux.
% B.type(str): type of BC;
% B.value(cell): boundary value.
%--- specify boundary condition
[B.index, Len] = fem1d_bc(x,bn_index,B.criterion);
% B_index(array): nonzero elements on each column are associated with one BC.
% Len(array): value of each entry is the number of nonzeroes on each column
%                in B_index.
t_initial      = domain.range(1);
t_final        = domain.range(2);
bn             = fem1d_bc_eval(x, B.index, Len, B.value, B.type, t_initial);
% bn.dir(array): first column contains Dirichlet boundary nodes,
%                secound column contains boundary values at t_initial;
% bn.neu(array): Neumann BC; bn.rob(array): Robin BC; bn.fre(array): free BC.
nodes_all = 1:n_nodes;
if isempty(bn.dir) == 1  
    nodes_nondir = nodes_all;  % nodes_nondir: non-Dirichlet nodes including both interior and non-Dirichlet boundary nodes
else
    nodes_nondir = setdiff(nodes_all, bn.dir(:,1));
end




fem_settings = struct(...
    'fem_mesh', fem_mesh,...  % FEM mesh
    'opt',      opt,...
    'filter',  filter,...
    'domain',  domain,... 
    'u',       u, ...     % u.func(func):  either u or unknown;    u.var(sym): symbolic variable of u.
    'f',       f,...      % f.func(func):  either f or unknown;    f.var(sym): symbolic variable of f.
    'ux',      ux,...     % ux.func(func): either du/dx or unknow; ux.var(sym):symbolic variable of ux
    'B',       B,...      % B.type(str): type of BC;  % B.value(cell): boundary value;  B_index(array): nonzero elements on each column are associated with one BC
    'Nonlfunc',Nonlfunc,...  
    'Len',     Len,...    % Len(array): value of each entry is the number of nonzeroes on each column
    't_initial',t_initial,... 
    't_final',  t_final,...
    'bn',       bn,...    % bn.dir(array): first column contains Dirichlet boundary nodes,
    'nodes_nondir',nodes_nondir, ...  % nodes_nondir: non-Dirichlet nodes including both interior and non-Dirichlet boundary nodes
    'n_gauss', n_gauss...             % Gaussian quadrature in the FE integration
    );
