function  burgers1d_fem_1traj(opt,domain,fem_settings,u0, filenames,saveON,test_ind)
% generate 1-trajectory solution by FEM with implicit time-forward (by Newton's method)
% Output: 
% - correlation matrix (i.e., the spatial covariance matrix, denote by kmat), 
%   computed by u*u'. Saved to local file. 
% - u0: the initial condition, which will be used to generate the
% trajectory again to compute the coefficients of the POD basis            % if the solver is expensive, save the whole trajectory data instead.  
%{
For this 1D FEM solver, mesh generating is very fast; the most expensive part is the Newton's solver
which needs to update the trilinear term (Jacobian) at each time step
%}
%--------------------------------------------------------------------------
% Updated version of FEM solver for Burgers equation
% Orginally from Zhu Wang
% modified by Changhong Mou, Fei Lu

File_shape = filenames.File_shape;
File_smm   = filenames.File_smm;
File_mesh  = filenames.File_mesh;
File_snap  =  filenames.File_snap;

File_snapshots_test = [File_snap,'test_',num2str(test_ind)];
if exist(File_snapshots_test,'file')
    fprintf('Using previous data for trajectory %i \n',test_ind); 
    return; 
end
    
%---- load FEM mesh
if exist([File_mesh,'.mat'], 'file') == 2
    % POD file does exist.
    load(File_mesh)    
    fprintf(1, ['Loading mesh data from ', char(File_mesh), '\n']);
else
    burgers1d_fem_mesh_mtraj(opt,domain,fem_settings, filenames);    
end

fem_mesh = fem_settings.fem_mesh;
filter   = fem_settings.filter;

%n_gauss  = fem_settings.n_gauss;
time_start = tic;
% %--- generate 1D mesh
% [x,e_conn,bn_index,fem_mesh] = fem1d_mesh(domain, elem_order);     % generate mesh
% x         = fem_mesh.x;        % x: X-coordinates;
% e_conn    = fem_mesh.e_conn;   % e_conn: element info
bn_index  = fem_mesh.bn_index;


n_nodes     = size(x,1);
n_elements  = size(e_conn,1);

%--- specify test problem
u         = fem_settings.u;
%f         = fem_settings.f;
ux        = fem_settings.ux;
%B         = fem_settings.B;   % B.type(str): type of BC;  % B.value(cell): boundary value.
% B_index(array): nonzero elements on each column are associated with one BC.
%Nonlfunc  = fem_settings.Nonlfunc;
%--- specify boundary condition
%Len       = fem_settings.Len;  % Len(array): value of each entry is the number of nonzeroes on each column

t_initial = fem_settings.t_initial;
t_final   = fem_settings.t_final;
%bn        = fem_settings.bn;
% bn.dir(array): first column contains Dirichlet boundary nodes,
%                secound column contains boundary values at t_initial;
% bn.neu(array): Neumann BC; bn.rob(array): Robin BC; bn.fre(array): free BC.

%nodes_nondir  = fem_settings.nodes_nondir;  % nodes_nondir: non-Dirichlet nodes including both interior and non-Dirichlet boundary nodes

% if length(u0) ~=n_nodes
%     u0 = get_IC(fem_settings);
% end



%--- integration preparation
%File_shape = fem1d_shape_gen(x,e_conn,n_gauss,File_shape);   % generate and store shape function info.
% File_shape: data which stores shape function info.
%[Mass, S, P]  = fem1d_assemble_bilinear(x,e_conn,n_gauss,File_shape,File_smm);  % generate mass and stiffness
% Mass(array): mass matrix; S1+S2(array): stiffness matrix; P: (hx, h)
%--- initialization of variables
n_save      = fix((t_final-t_initial)/domain.tsave)+1;
u_save      = zeros(n_nodes,n_save); u_exact = u_save;
u0(2:end-1) = (Mass(2:end-1,2:end-1)+filter.delta.^2*S(2:end-1,2:end-1))\(Mass(2:end-1,2:end-1)*u0(2:end-1));
u_save(:,1) = u0; % @ t=t_initial
u_exact(:,1)= u0;
count_s     = 1;
%--- calculate FE load vector
f_time_dep = sum(strcmp('t',f.var));% determin if problem is time-dependent
f_FE   = fem1d_load(x,e_conn,n_gauss,f,t_initial,File_shape);% FE load at t(n)
f_BE   = f_FE;                        % initial FE load at t(n+1)
%---- convert string to function handle
% BC_handle = str2func(B.value{1}); %for B.C.
if opt.rhs~=0                    %for nonlinear functions
    ker_nonlfunc_grad = str2func(Nonlfunc.grad_str);
    ker_nonlfunc      = str2func(Nonlfunc.str);
else
    ker_nonlfunc_grad = [];
    ker_nonlfunc      = [];
end


%% ___________________________% numerical simulation: time integrations
for i_t = t_initial: domain.t: t_final-domain.t
    time = i_t+ domain.t;
    if f_time_dep~=0         % if time-dependent then refresh FE load at t(n+1)
        f_BE   = fem1d_load(x,e_conn,n_gauss,f,time,File_shape);
    end
    %     Neu_FE = FEM_function_value(BC_handle,'t',0,i_t);  % Neumann BC at x=0 when t= t(n)
    %     Neu_BE = FEM_function_value(BC_handle,'t',0,time);% Neumann BC at x=0 when t= t(n+1)
    %     Neu_BE = (5e+4)*time^3*exp(-15*time);% Neumann BC at x=0 when t= t(n+1)
    if opt.theta<0
        %- semi-implicit method
        [U1] = Burgers1D_FEM_semi...
            (opt,domain,x,e_conn,n_gauss,Mass,S,P, ...
            u0,f_FE,f_BE,File_shape,nodes_nondir,bn,...
            ker_nonlfunc_grad,ker_nonlfunc);   % Need to be updated here!
    else
        %- theta method
        [U1] = burgers1d_fem_newton...
            (opt,x,e_conn, n_gauss, domain, Mass,S,P, ...
            u0,f_FE,f_BE,File_shape,nodes_nondir,bn,...
            ker_nonlfunc_grad,ker_nonlfunc);
        
        %------ add the filter steps:   ======  why this septs?
        if filter.op==1
            U1(2:end-1) = (Mass(2:end-1,2:end-1)+...
                filter.delta.^2*S(2:end-1,2:end-1))\(Mass(2:end-1,2:end-1)*U1(2:end-1));
        end
        
    end
    u0=U1;
    
    num_temp = (time - t_initial)/domain.tsave;
    if ( abs(num_temp - round(num_temp))< 10^(-8))
        count_s = count_s + 1;
        u_save(:,count_s) = U1;
    end
    if f_time_dep~=0
        f_FE = f_BE;
    end
    
end
cputime = toc(time_start);
fprintf(1, ['Time elapsed is ',num2str(cputime),'\n']);


%% %____________________________% postprocess of simulation

%--- set output
outputInfo = filenames.outputInfo;
fig_num   = outputInfo.fig_num;
fig_evo_p = outputInfo.fig_evo_p;
fig_evo   = outputInfo.fig_evo;
cal_err   = outputInfo.cal_err ;              % output error values to screen
save_data = outputInfo.save_data;

%--- save data

%----
% save POD and snapshot correlation matrix and initial condition at each time while the mesh and time
Ms = Mass;
[k_mat] = burgers1d_k_mat_fun(u_save,Ms);

if save_data == 1
    save(File_snapshots_test,'k_mat',...
        'u0','u', 'ux')
    fprintf(1, ['Snapshots have been stored to ', char(File_snapshots_test), '\n']);
end

%--- Output to screen
fprintf(1, ['Time elapsed for simulations is ', num2str(cputime), '\n']);
if fig_evo_p == 1 % output process of evolution to screen
    figure(fig_num),
    for i = 1:size(u_save,2)
        x_cord = 1:length(x);
        plot( x(x_cord), u_save(x_cord,i)),
        pause(.1);
    end
    fig_num = fig_num + 1;
end
if fig_evo == 1
    skip = 1;
%     [XX,TT]=meshgrid(x, t_initial:domain.tsave:t_final);
%     figure(fig_num)
%     mesh(XX, TT, u_save');
%     xlabel('x'); ylabel('t');
    [XX,TT]=meshgrid(x, t_initial:domain.tsave:t_final);
    figure(fig_num)
    [~,h]=contourf(XX, TT, u_save',100);
    set(h, 'edgecolor','none');
    set(gca,'FontName','Times','fontsize',22)

    %title(sprintf('Finite Element Nodes=%i ',n_nodes));
    colormap jet; colorbar
    xlabel('x'); ylabel('t');
    
    %print('-depsc', File_fig)
    fig_num = fig_num + 1;
end
%--- calculate errors and output to screen
if cal_err == 1
    Err_max    = zeros(n_save, 1);       Err_L2 = Err_max;
    Err_H1_semi= Err_max;                Err_H1 = Err_max;
    for i = 1: domain.calerr_skip: n_save
        U1 = u_save(:,i);
        time = t_initial+ (i-1)*domain.tsave*domain.calerr_skip;
        [Err_max(i), Err_L2(i), Err_H1_semi(i), Err_H1(i)] = FEM_oned_error...
            (x, e_conn, U1, u, ux, time, File_shape);
        if i==n_save
            err_max_avg   = sqrt( sum(Err_max.^2)/n_save );
            err_L2_avg    = sqrt( sum(Err_L2.^2)/n_save );
            err_H1_semi_avg = sqrt( sum(Err_H1_semi.^2)/n_save );
            err_H1_avg    = sqrt( sum(Err_H1.^2)/n_save );
            fprintf(1, ['Time average C0 error square is ', num2str(err_max_avg), '\n']);
            fprintf(1, ['Time average L2 error square is ', num2str(err_L2_avg), '\n']);
            fprintf(1, ['Time average H1 error square is ', num2str(err_H1_avg), '\n']);
        end
    end
end


