function [u1] = burgers1d_fem_newton(opt, x, e_conn,n_gauss, domain, Mass, S, P, ...
    u0, f0, f1, filename_shape, nodes_nondir, bn, ...
    ~, ker_nonlfunc)


global t_initial t_final
% global n_nodes n_gauss
% global domain
%--------------------------------------------------------------------------
% This routine use theta method for time discretization,
% solve the nonlinear equation by Newton's method
%--------------------------------------------------------------------------
RESID     = [ ] ;   NORM_newton = [ ];
converged = 0;      diverged =0;       iterations=0;
max_iterations  = 20;
tol_step  = 1.d-10;
tol_resid = 1.d-10;

n_nodes   = size(u0,1);
utest     = nodes_nondir;     % related to number of test functions
utral     = nodes_nondir;     % related to number of trial functions
N1        = length(utest);
N3        = length(utral);

RInd1     = 1 : N1;
CInd1     = 1 : N3;    % column index relates to number of trial functions

%{
%----test differential filter
opt.r = 6*domain.h;
%--- step 1, at t=t0, u0 is given, cal v0
v0 = diff_filter(u0,opt.r,Mass,S,nodes_nondir,bn.dir(:,1));
%--- step 2, at t=t0, u0 is given, cal v0bar
v0bar = diff_filter(v0,opt.r,Mass,S,nodes_nondir,bn.dir(:,1));
% plot(x,u0,x,v0,'ro-',x,v0bar,'kx-',x,v0star,'g^-')
%--- step 3, solve AD equation (semi-implicit)
v0star= 2*v0-v0bar;
N=10;
v0star1 = AD_diff_filter(v0,opt.r,N,Mass,S,nodes_nondir,bn.dir(:,1));
plot(x,u0,x,v0,'ro-',x,v0bar,'kx-',x,v0star,'g^-',x,v0star1,'cs-')
pause(.05)
%}

Mat_A     = Mass;
Mat_B     = opt.parm_e*S; % opt.parm_e is viscosity

A_f       = Mat_A + opt.theta*domain.t*Mat_B;

rhs_f     = (-Mat_A+(1-opt.theta)*domain.t*Mat_B)*u0...
    -domain.t*(opt.theta*f1+(1-opt.theta)*f0);

 while (~converged && ~diverged)
    iterations = iterations + 1;
    if isempty(ker_nonlfunc)==1
        d_nonlin = sparse(n_nodes,n_nodes);
        Nh       = sparse(n_nodes, 1);
    end
    [T1, T2] = fem1d_burgers_assemble_trilinear...
        (x,e_conn,n_gauss,u0,filename_shape);
    d_nonlin = d_nonlin+(T1+T2);
    Nh       = Nh+(T1*u0);
    if iterations == 1
        rhs_f = rhs_f + (1-opt.theta)*domain.t*Nh;
    end
     
    Ac  = A_f+ opt.theta*domain.t*(d_nonlin);
    bc  = -( A_f*u0 + opt.theta*domain.t*Nh+ rhs_f );
    A   = Ac(utral,utral);
    b   = bc(utral,1); % -Ac(utest,bn.dir(:,1))*zeros(size(bn.dir(:,1),1)) % since it is newton step
    %--------------------------------
    Newton_step = A\b;
    u0(utral,1) = u0(utral,1) + Newton_step(CInd1,1);
    
    norm_step = norm(Newton_step,2);
    norm_u    = norm(u0, 2);
    norm_resid = norm(A*Newton_step-b,2);
    if ( iterations==1 )
        norm_resid0 = max(norm_resid,1e-20);
    end
    converged = (norm_resid/norm_resid0 < tol_resid || ...
        norm_resid < tol_resid           ) && ...
        norm_step/norm_u    < tol_step ;
    
    diverged = (iterations>max_iterations);
    RESID = [ RESID, norm_resid];
    NORM_newton = [NORM_newton, norm_step];
    
    if diverged==1
        fprintf(1, 'Newton solver diverges\n');
    end
end %(while)

u1=u0;

