% domain.interval = [0, 1];
% elem_order = 1;
% domain.h = 1/2^10;
% domain.range = [0, 1];
% domain.t = 1e-3;
% domain.tsave = 10*domain.t;
% domain.calerr_skip = 1;
% 
% [x,e,bn] = FEM_oned_mesh(domain, elem_order)


function [Mass, S] = Test(opt, elem_order, domain)
fprintf(1,'===== New run starts =====\n');
time_start = tic;
%========= user-defined paremeters
%--- specify test problem
if nargin ==0
    opt.es     = 0;          % choose test problem
    opt.rhs    = 0;          % choose equations & rhs
    opt.bc     = 0;          % choose boundary conditions
    opt.ic     = 3;          % choose initial conditions
    opt.theta  = 0.5;          % choose theta value, if negative, semi-implicit
    domain.interval = [0, 1];% spatial interval
    elem_order = 1;          % choose the order of element
    domain.h   = 1/2^10;      % choose mesh size on spatial space
    domain.range = [0, 1];   % time range
    domain.t   = 1e-3;       % choose time step on tempral space
    domain.tsave = 10*domain.t;
    domain.calerr_skip = 1;  % skip some steps when calculate errors
end
opt.parm_e = 0.01;

%--- set numerical discretization
n_gauss    = 5;              % Gaussian quadrature in the FE integration
%--- set output
fig_num    = 1;
fig_evo_p  = 1;
fig_evo    = 1;
cal_err    = 0;              % output error values to screen
save_data  = 1;              % choose whether saving data or not
%--- generate 1D mesh
[x,e_conn,bn_index] = FEM_oned_mesh(domain, elem_order);     % generate mesh
% x: X-coordinates; e_conn: element info
n_nodes     = size(x,1);
n_elements  = size(e_conn,1);
%--- specify test problem
[u,f,ux,B,Nonlfunc]  = Burgers1D_FEM_setting(opt,domain);
% u.func(func):  either u or unknown;    u.var(sym): symbolic variable of u.
% f.func(func):  either f or unknown;    f.var(sym): symbolic variable of f.
% ux.func(func): either du/dx or unknow; ux.var(sym):symbolic variable of ux.
% B.type(str): type of BC;
% B.value(cell): boundary value.
%--- specify boundary condition
[B.index, Len] = FEM_BC(x,bn_index,B.criterion);
% B_index(array): nonzero elements on each column are associated with one BC.
% Len(array): value of each entry is the number of nonzeroes on each column
%                in B_index.
t_initial      = domain.range(1);
t_final        = domain.range(2);
bn             = FEM_boundary_value(x, B.index, Len, B.value, B.type, t_initial);
% bn.dir(array): first column contains Dirichlet boundary nodes,
%                secound column contains boundary values at t_initial;
% bn.neu(array): Neumann BC; bn.rob(array): Robin BC; bn.fre(array): free BC.
nodes_all = 1:n_nodes;
if isempty(bn.dir) == 1
    nodes_nondir = nodes_all;
else
    nodes_nondir = setdiff(nodes_all, bn.dir(:,1));
end
% nodes_nondir: non-Dirichlet nodes including both interior and non-Dirichlet boundary nodes
%--- specify initial condition
if opt.ic == 3
    u0 = Burgers1D_IC(x);
else
    u0 = FEM_IC(opt.ic,x,bn,u,nodes_nondir,t_initial);
end
% u0(array): initial condition vector.
%--- set file names
Folder_save = '../Data/';
Filename0   = ['Burgers1D_FEM_',num2str(opt.es),num2str(opt.rhs),...
    '_P', num2str(elem_order), '_h', num2filename(domain.h),...
    '_dt', num2str(num2filename(domain.t)),...
    '_cta', num2str(num2filename(opt.theta)), '_'];
File_shape  = [Folder_save, Filename0];
File_smm    = [Folder_save, Filename0, 'FEIC_SMM'];
File_dns    = [Folder_save, Filename0,'dns'];
File_fig    = ['../Figures/', Filename0, 'dns'];
%--- integration preparation
File_shape = FEM_gen_shape(x,e_conn,n_gauss,File_shape);   % generate and store shape function info.
% File_shape: data which stores shape function info.
[Mass, S, P]  = FEM_assemble_bilinear_1D(x,e_conn,n_gauss,File_shape,File_smm);  

end