function [bn_index_new, Len] = fem1d_bc(x,bn_index,criterion)
%--------------------------------------------------------------------------
% Given total boundary nodes generated by John's codes, we
% specify different types of boundary nodes basing on
% "criteria".
% each column of "bn_index_new" contains
% the boundary nodes in a criterion
%
% Inputs
%   criterion: boundary criterion
%       - Full boundary: total boundary nodes
%       - others: the rest of boundary after specifying certain types
%       (always put to the end)
%   bn_index: indices for boundary nodes;
% Outputs
%   bn_index_new: new boundary nodes index; each column
%   contains nodes belong to one of the criteria;
%   Len: the number of criteria;
%--------------------------------------------------------------------------
if size(x,2)~=1
    error('Not 1D \n');
end
bn_x = x(bn_index,:); % coordinates of all boundary nodes
if strcmp(criterion, {'Full boundary'})==1 % BC includes all nodes on edges
    bn_index_new = bn_index;
    Len = length(bn_index);
else
    x     = bn_x(:,1);
    Len_c = length(criterion); % number of criteria
    Len   = zeros(Len_c,1);    % nodes corresponded to each criteria
    Index_r =1;
    Temp_boundary_index(Len_c).mat = [];
    for i=1:Len_c
        if strcmp(criterion(i),{'Others'})==1
            Mesh_boundary = bn_index;
            for k=1:i-1
                Mesh_boundary = Boundary_exclusive(Mesh_boundary,Temp_boundary_index(k).mat);
            end
            Temp_boundary_index(i).mat = Mesh_boundary;
        else
            bound_str = char(criterion(i));
            Temp_boundary_index(i).mat = bn_index( eval(bound_str) );
            Index_r = Index_r+1;
            Len(i) = length(Temp_boundary_index(i).mat);
        end
        
        bn_index_new = zeros(max(Len), Len_c);
        for i = 1:Len_c
            bn_index_new(1:Len(i),i)=Temp_boundary_index(i).mat;
        end
    end
end

end


function [Array] = Boundary_exclusive(Array,Exclusive)
for i = 1:length(Exclusive)
    Array = Array( Array~=Exclusive(i) );
end
end



