function [bn] = fem1d_bc_eval(x, B_index, Len, B_value, B_type, t)
%--------------------------------------------------------------------------
% Written by Zhu Wang 06-07-2011
% If the compontent of B_value is empty, we don't enforce boundary nodes on
% that boundary. Otherwise, nonempty B_value contains boundary info.: each
% component of B_value can be a function handle or a constant.
% If it is a function handle, we evaluate it at given boundary nodes;
% otherwsie, it is a constant, we just set boundary values to be this constant.
%--------------------------------------------------------------------------
if size(x,2)~=1
    error('Not 1D \n');
end

bn.dir  = []; % Dirichlet BC
bn.neu  = []; % Neumann BC
bn.rob  = []; % Robin BC
bn.fre  = []; % Free BC

for k = 1: length(find(B_type=='D'))
    dir_index = B_index(1:Len(k), k);
    if isa(B_value{k},'double') % design constant B_value on this boundary
        dir_value = ones(length(dir_index),1).*B_value{k};
    else
        if isa(B_value{k},'char')==1
            BC_handle = str2func(B_value{k});
        elseif isa(B_value{k},'function_handle')==1
            BC_handle = B_value{k};
        else
            error('NA')
        end
        arglist   = Matlab_arglist(BC_handle);
        dir_value = FEM_function_value(BC_handle, arglist, x(dir_index,:),t);
    end
    dir_mat =[dir_index, dir_value];
    bn.dir = [bn.dir; dir_mat];
end

for k = 1: length(find(B_type=='N'))
    neu_index = B_index(1:Len(k), k);
    if isa(B_value{k},'double') % design constant B_value on this boundary
        neu_value = ones(length(neu_index),1).*B_value{k};
    else
        if isa(B_value{k},'char')==1
            BC_handle = str2func(B_value{k});
        elseif isa(B_value{k},'function_handle')==1
            BC_handle = B_value{k};
        else
            error('NA')
        end
        arglist   = Matlab_arglist(BC_handle);
        neu_value = FEM_function_value(BC_handle, arglist, x(neu_index,:),t);
    end
    neu_mat =[neu_index, neu_value];
    bn.neu = [bn.neu; neu_mat];
end

for k = 1: length(find(B_type=='R'))
    rob_index = B_index(1:Len(k), k);
    if isa(B_value{k},'double') % design constant B_value on this boundary
        rob_value = ones(length(rob_index),1).*B_value{k};
    else
        if isa(B_value{k},'char')==1
            BC_handle = str2func(B_value{k});
        elseif isa(B_value{k},'function_handle')==1
            BC_handle = B_value{k};
        else
            error('NA')
        end
        arglist   = Matlab_arglist(BC_handle);;
        rob_value = FEM_function_value(BC_handle, arglist, x(rob_index,:),t);
    end
    rob_mat =[rob_index, rob_value];
    bn.rob = [bn.rob; rob_mat];
end

for k = 1: length(find(B_type=='F'))
    fre_index = B_index(1:Len(k), k);
    bn.fre = [bn.fre; fre_index];
end






