function [Trilinear1, Trilinear2]= fem1d_burgers_assemble_trilinear...
                (x, e_conn,n_gauss, u, filename_shape)
            
%=====================================
% 10/31/2011
% Developed by Zhu Wang basing on John Burkdart's codes
% To evaluate the trilinear term
%
% Inputs:
%       x - coordinates matrix
%       e_conn - connective matrix
%       n_gauss -  # of Gauss points
%       u - state variable vector 
% Outputs:
%       Trilinear1 (sparse) - (h*u_x, h)
%       Trilinear2 (sparse) - (h_x*u, h)
%
%======================================

[N,          dim    ] = size(x     );
[n_elements, nel_dof] = size(e_conn);
load(filename_shape,'W_w_g','W_phi','W_p_x')
% Due to memory limitations, we will fill the mass matrix by integrating
% over partitions of elements.  The constant (max_elem_per_partition)
% should be adjusted based on available memory.
max_elem_per_partition = 5000;

n_part = floor( n_elements / (max_elem_per_partition+1) ) + 1;
elem_segment = floor( linspace(0,n_elements,n_part+1) );
max_part_size = max( diff( elem_segment ) );

if (dim==1)
    [r,w] = fem1d_gauss_gen(n_gauss);
    one = ones(size(w));
    for n_pt = 1:n_part
        II = zeros( max_part_size*nel_dof^2,1 );
        JJ = zeros( max_part_size*nel_dof^2,1 );
        XX = zeros( max_part_size*nel_dof^2,1 );
        YY = zeros( max_part_size*nel_dof^2,1 );
        entry_counter = 0;
        
        for n_el=elem_segment(n_pt)+1:elem_segment(n_pt+1)
            nodes_local            = e_conn(n_el,:);
            x_local                = x(nodes_local,:);
%             [~, w_g, phi, p_x] = FEM_oned_shape(x_local,r,w);
            w_g      = W_w_g(:,n_el);
            phi      = W_phi(:,(n_el-1)*nel_dof+1:n_el*nel_dof);
            p_x      = W_p_x(:,(n_el-1)*nel_dof+1:n_el*nel_dof);
            
            u_loc    = u(nodes_local, 1);
            tri_loc1 = fem1d_bilinear(p_x*u_loc, phi, phi, w_g );  %(h*u_x, h)
            tri_loc2 = fem1d_bilinear(phi*u_loc, p_x, phi, w_g);   %(h_x*u, h)
            
            %  Assemble into the global system matrix
            for i=1:nel_dof
                for j=1:nel_dof
                    entry_counter = entry_counter + 1;
                    II(entry_counter) = nodes_local(i);
                    JJ(entry_counter) = nodes_local(j);
                    XX(entry_counter) = tri_loc1(i,j);
                    YY(entry_counter) = tri_loc2(i,j);
                end
            end
            
        end % n_el loop
        
        if ( n_pt==1 )
            Trilinear1 = sparse( II(1:entry_counter), JJ(1:entry_counter),...
                XX(1:entry_counter),...
                N, N );
            Trilinear2 = sparse( II(1:entry_counter), JJ(1:entry_counter),...
                YY(1:entry_counter),...
                N, N );
        else
            Trilinear1 = Trilinear1 + ...
                sparse( II(1:entry_counter), JJ(1:entry_counter),...
                XX(1:entry_counter),...
                N, N );
            Trilinear2 = Trilinear2 + ...
                sparse( II(1:entry_counter), JJ(1:entry_counter),...
                YY(1:entry_counter),...
                N, N );            
        end
        
    end % n_pt loop
else 
    fprintf(1, 'This function only supports 1D cases \n');
end 
