function  [Err_max, Err_L2, Err_H1_semi, Err_H1] = fem1d_error...
    (x, e_conn, u_h, u, ux, t, filename_shape)
%--------------------------------------------------------------------------
% calculate errors between finite element solution and the exact solution
% on a finite element mesh
%--------------------------------------------------------------------------

if isempty(u.func)==1
    error('Exact solution is unknown, consider to use FEM_hH_error.m');
end
dim            = size(x , 2);
N              = max(e_conn(:)); % Since it may use bubble function
[n_elements, nel_dof]= size(e_conn);

%--- initialization
load(filename_shape,'W_x_g','W_w_g','W_phi','W_p_x')

Err_max        = max(abs(FEM_function_value(u.func, u.var, x, t)-u_h));
Err_L2         = 0;
Err_H1_semi    = 0;
Err_H1         = 0;

if (dim==1)
    for n_el = 1:n_elements
        nodes_local              = e_conn(n_el,:);
        
        x_g     = W_x_g(:,n_el);
        w_g     = W_w_g(:,n_el);
        phi     = W_phi(:,(n_el-1)*nel_dof+1:n_el*nel_dof);
        phi_x   = W_p_x(:,(n_el-1)*nel_dof+1:n_el*nel_dof);
        %         [x_g, w_g, phi, p_x]= FEM_oned_shape(x_local,r,w);
        u_h_local                = u_h(nodes_local,:);
        u_h_local_g              = phi*u_h_local;
        u_e_local_g              = FEM_function_value(u.func, u.var, x_g, t);
        
        ux_h_local_g             = phi_x*u_h_local;
        ux_e_local_g             = FEM_function_value(ux.func,ux.var,x_g, t);
        
        Err_L2 = Err_L2 + w_g'*(u_e_local_g-u_h_local_g).^2;   %
        Err_H1_semi = Err_H1_semi + w_g'*(ux_e_local_g-ux_h_local_g).^2;
    end
    Err_H1      = Err_L2+Err_H1_semi;
    Err_L2      = sqrt(Err_L2);
    Err_H1_semi = sqrt(Err_H1_semi);
    Err_H1      = sqrt(Err_H1);
end

