function [x,e_conn,bn,fem_mesh]=fem1d_mesh(domain,elem_order)
%--------------------------------------------------------------------------
% Generate 1 D Mesh given spatial 'interval' and 'n_elements' of elements
% Output coordinates 'x', connective matrix 'e_conn'; 
% 'node' interior nodes, mesh size 'h'
%--------------------------------------------------------------------------
n_elements = round( (domain.interval(2)-domain.interval(1))/domain.h ); 
n_nodes = elem_order*n_elements + 1;
x     = linspace( domain.interval(1), domain.interval(2), n_nodes );   
x     = x';

e_conn = zeros(n_elements,elem_order+1);
for e  = 1:n_elements
    e0 = (e-1)*(elem_order);
    e_conn(e,:)= e0+(1:elem_order+1);
end
     
bn     = [1; n_nodes]; % boundary nodes

fem_mesh = struct(...
    'domain',     domain,...
    'elem_order', elem_order,... 
    'n_elements', n_elements,...
    'n_nodes',    n_nodes,...
     'x',         x,...             % x: X-coordinates;
     'e_conn',    e_conn,...        % e_conn: element info
     'bn_index',  bn ...       
    ); 