function [T] = pod1d_assemble_tensor_t(POD_u, x, e_conn, n_gauss, filename_shape)

[N         , q      ] = size(POD_u );
[n_elements, nel_dof] = size(e_conn);
dim                   = size(x, 2);
%--- initialization
load(filename_shape,'W_w_g','W_phi','W_p_x')

Tux(q).loc = []; T(q).ux = []; XT(q).ux = [];
% Due to memory limitations, we will fill the mass matrix by integrating
% over partitions of elements.  The constant (max_elem_per_partition)
% should be adjusted based on available memory.
max_elem_per_partition = 5000;

n_part = floor( n_elements / (max_elem_per_partition+1) ) + 1;
elem_segment = floor( linspace(0,n_elements,n_part+1) );
max_part_size = max( diff( elem_segment ) );

if (dim==1)
    [r,w] = fem1d_gauss_gen(n_gauss);
    %     one = ones(size(w));
    for n_pt = 1:n_part
        II = zeros( max_part_size*nel_dof^2,1 );
        JJ = zeros( max_part_size*nel_dof^2,1 );
        for i=1:q
            XT(i).ux = sparse( max_part_size*nel_dof^2,1 );
        end
        entry_counter = 0;
        
        for n_el=elem_segment(n_pt)+1:elem_segment(n_pt+1)
            nodes_local            = e_conn(n_el,:);
            
            w_g     = W_w_g(:,n_el);
            phi     = W_phi(:,n_el*2-1:n_el*2);
            phi_x   = W_p_x(:,n_el*2-1:n_el*2);
            
            for k=1:q
                podu_k = phi*POD_u(nodes_local,k);
                Tux(k).loc = fem1d_bilinear(podu_k, phi_x, phi, w_g );
            end
            
            %  Assemble into the global system matrix
            entry_counter = entry_counter + 1;
            II  (entry_counter:entry_counter+nel_dof^2-1) = ...
                reshape((nodes_local(1:nel_dof)'*ones(1,nel_dof))',nel_dof^2,1);
            JJ  (entry_counter:entry_counter+nel_dof^2-1) = ...
                reshape(nodes_local(1:nel_dof)'*ones(1,nel_dof),nel_dof^2,1);
            
            for k=1:q   % recomputing the local arrays extra times, should break out
                % this part of the calculation...
                XT(k).ux(entry_counter:entry_counter+nel_dof^2-1) = ...
                    reshape(Tux(k).loc',nel_dof^2,1);
            end
            entry_counter = entry_counter + nel_dof^2 -1;
            
        end % n_el loop
        
        if ( n_pt==1 )
            
            for k=1:q
                T(k).ux = sparse( II(1:entry_counter), JJ(1:entry_counter),...
                    XT(k).ux(1:entry_counter),...
                    N, N );
            end
        else
            for k=1:q
                T(k).ux = T(k).ux + sparse( II(1:entry_counter), JJ(1:entry_counter),...
                    XT(k).ux(1:entry_counter),...
                    N, N );
            end
        end
        
    end % n_pt loop
else
    fprintf(1, 'This function only supports 1D cases \n');
end

