function  fem_settings = burgers1d_fem_mesh_mtraj_v2(opt,domain,fem_settings, filenames)
% generate 1-trajectory solution
%{
 FL remarks to CM:
- will it be faster if we pass parameters, mesh as inputs?
CM:
- for 1D, mesh generating is very fast, so it will not remarkably save
time; for this FEM solver, the most expensive part is the Newton's solver
which needs to update the trilinear term (Jacobian) at each time step
%}
%--------------------------------------------------------------------------
% Updated version of FEM solver for Burgers equation
% Orginally from Zhu Wang
% modified by Changhong Mou, Fei Lu

File_shape = filenames.File_shape;
File_smm   = filenames.File_smm;
File_mesh   = filenames.File_mesh;
%--------------------------------------------------------------------------
fem_mesh = fem_settings.fem_mesh;

n_gauss  = fem_settings.n_gauss;
% %--- generate 1D mesh
% [x,e_conn,bn_index,fem_mesh] = fem1d_mesh(domain, elem_order);     % generate mesh
x         = fem_mesh.x;        % x: X-coordinates;
e_conn    = fem_mesh.e_conn;   % e_conn: element info

%--- specify test problem
f         = fem_settings.f;
B         = fem_settings.B;   % B.type(str): type of BC;  % B.value(cell): boundary value.
% B_index(array): nonzero elements on each column are associated with one BC.
Nonlfunc  = fem_settings.Nonlfunc;
%--- specify boundary condition
Len       = fem_settings.Len;  % Len(array): value of each entry is the number of nonzeroes on each column

t_initial = fem_settings.t_initial;
t_final   = fem_settings.t_final;
bn        = fem_settings.bn;
% bn.dir(array): first column contains Dirichlet boundary nodes,
%                secound column contains boundary values at t_initial;
% bn.neu(array): Neumann BC; bn.rob(array): Robin BC; bn.fre(array): free BC.

nodes_nondir  = fem_settings.nodes_nondir;  % nodes_nondir: non-Dirichlet nodes including both interior and non-Dirichlet boundary nodes

% if length(u0) ~=n_nodes
%     u0 = get_IC(fem_settings);
% end



%--- integration preparation
File_shape = fem1d_shape_gen(x,e_conn,n_gauss,File_shape);   % generate and store shape function info.
% File_shape: data which stores shape function info.
[Mass, S, P]  = fem1d_assemble_bilinear(x,e_conn,n_gauss,File_shape,File_smm);  % generate mass and stiffness
% Mass(array): mass matrix; S1+S2(array): stiffness matrix; P: (hx, h)

fem_settings.tN       = length(t_initial: domain.t: t_final);   % number of time steps
fem_settings.Mass     = Mass;
fem_settings.S        = S;   % stiff matrix
fem_settings.P        = P;
fem_settings.File_shape     = File_shape;

%--- set output
outputInfo = filenames.outputInfo;
%save_data = outputInfo.save_data;

%--- save data

% save mesh data, save once
%if save_data == 1
    save(File_mesh, 'x', 'e_conn', 'Mass', 'S','P',...
        'opt', 'n_gauss', 'f', ...
        'B', 'bn', 'Len','Nonlfunc','nodes_nondir',...
        'domain','File_shape','File_smm')
    fprintf(1, ['FEM mesh data (for multiple runs) have been stored to ', char(File_mesh), '\n']);
    
%end