function [u1] = Burgers1D_ROM_Newton...
    (opt, domain, Mr, Sr, ...
    ur0, fr0, f1, Tensor_rom)
%--------------------------------------------------------------------------
% This routine use theta method for time discretizati on,
% solve the nonlinear equation by Newton's method in the ROM parts
% Jan 24, 2018 by Mou; orginally from Zhu Wang
%--------------------------------------------------------------------------
RESID     = [ ] ;   NORM_newton = [ ];
converged = 0;      diverged =0;       iterations=0;
max_iterations  = 20;
tol_step  = 1.d-10;
tol_resid = 1.d-10;
n_nodes   = size(ur0,1);
[~,~,R] = size(Tensor_rom);
Mat_A     = Mr;
Mat_B     = opt.parm_e*Sr; % opt.parm_e is viscosity

Ar_f       = Mat_A + opt.theta*domain.t*Mat_B;
rhs_f     = (-Mat_A+(1-opt.theta)*domain.t*Mat_B)*ur0...
    -domain.t*(opt.theta*f1+(1-opt.theta)*fr0);

 while (~converged && ~diverged)
    iterations = iterations + 1;
    Jr = zeros(R,R);
    Nh = zeros(R,1);
   for i =1:R
    temp = (ur0)'*(Tensor_rom(:,:,i)+Tensor_rom(:,:,i)');
    Jr(i,:) = temp ;
    Nh(i) = (ur0)'*Tensor_rom(:,:,i)*ur0; %
    end
    

    if iterations == 1
        rhs_f = rhs_f + (1-opt.theta)*domain.t*Nh;
    end
     
    ArC  = Ar_f+ opt.theta*domain.t*(Jr);
    brc  = -( Ar_f*ur0 + opt.theta*domain.t*Nh+ rhs_f );
    A   = ArC;
    b   = brc; % -Ac(utest,bn.dir(:,1))*zeros(size(bn.dir(:,1),1)) % since it is newton step
    %--------------------------------
    Newton_step = A\b;
    ur0 = ur0+ Newton_step;
    
    norm_step = norm(Newton_step,2);
    norm_u    = norm(ur0, 2);
    norm_resid = norm(A*Newton_step-b,2);
    if ( iterations==1 )
        norm_resid0 = max(norm_resid,1e-20);
    end
    converged = (norm_resid/norm_resid0 < tol_resid || ...
        norm_resid < tol_resid           ) && ...
        norm_step/norm_u    < tol_step ;
    
    diverged = (iterations>max_iterations);
    RESID = [ RESID, norm_resid];
    NORM_newton = [NORM_newton, norm_step];
    
    if diverged==1
        fprintf(1, 'Newton solver diverges\n');
    end
end %(while)

u1=ur0;

