%--------------------------------------------------------------------------
% Galerkin ROM Code for 1D Burger's equation
% C. Mou 02/16/2020; Orginally from Zhu Wang
%--------------------------------------------------------------------------
clc; clear; close all
addpath ../ROM_FUN/
%addpath ../../FEM/FEM_FUN/

format long e

global W_w_g W_phi W_p_x
global t_initial t_final
global n_nodes n_gauss
global domain
%------------------------------------------
%--- specify test problem
opt.es     = 0;          % choose test problem %%just for numbering of the case
opt.rhs    = 0;          % choose equations & rhs
opt.bc     = 0;          % choose boundary conditions
opt.ic     = 3;          % choose initial conditions
opt.theta  = 1;          % choose theta value, if negative, semi-implicit
domain.interval = [0, 1];% spatial interval
elem_order = 2;          % choose the order of element
domain.h   = 1/2^9;      % choose mesh size on spatial space
domain.range = [0, 1];   % time range
domain.t   = 1e-4;       % choose time step on tempral space
domain.tsave = domain.t;
domain.calerr_skip = 1;  % skip some steps when calculate errors

opt.parm_e = 1e-3; % viscosity

%------------------------------------------
Folder_read = '../../FEM_Data/';
[File_shape,File_smm,File_dns] = burgers1d_fom_dir_gen(Folder_read,opt,domain,elem_order);

load(File_dns)
load(File_smm)
load(File_shape)

%------------------------------------------
%---- generate POD basis functions
Ms = Mass; % L^2 space
% Ms = Mass+S; % H^1 space

[pod_all,diag_S,d,eig_cum,eigen_cum_ratio] = pod_basis_gen_fun...
    (u_save,Ms);


r = 5;

pod_u =pod_all(:,1:r);

[Td_0x,~] = pod1d_trilinear_fun(pod_all,r,x, e_conn,n_gauss);
Tensor_rom  = Td_0x(1:r,1:r,1:r);

%-----------------
t_final = 1;
t_initial= 0;
n_save     = fix((t_final-t_initial)/domain.tsave);
u_r    = zeros(r,n_save);

u0 = u_save(:,1);
%-----------------
Mr=pod_u'*Mass*pod_u; %------- Mass matrix in ROM
Sr=pod_u'*S*pod_u;  %------- Stiffness matrix in ROM

u_r0=Mr\pod_u'*Mass*u0; % ------- ROM projection of the initial condition
u_proj_r = Mr\pod_u'*Mass*u_save; % ------- ROM projection of the snapshots

u_r(:,1)=u_r0;

save_index = 1;

%%
f0 = zeros(r,1);
f1 = zeros(r,1);
for k=1:n_save
    u_r1=Burgers1D_ROM_Newton(opt, domain, Mr, Sr, ...
        u_r0, f0, f1, Tensor_rom);
    u_r0=u_r1;
    if rem(k,save_index)==0
        u_r(:,k/save_index+1)=u_r1;
    end
end
%%

u_rom=pod_u*u_r;
u_fom = pod_u*u_proj_r;

%-----Caculate the error btw  FEM and POD ROM
Err_L2=zeros(n_save,1); % L2 error at each time step
Err_H1 = Err_L2;
Err_max = Err_L2; Err_H1_semi = Err_L2;
for i=1:n_save
    [Err_max(i),Err_L2(i),Err_H1_semi(i),Err_H1(i)]=...
        pod1d_error_fun(Mass,S,u_fom(:,i),u_rom(:,i));
end
Err_L2_avg=sqrt(sum(Err_L2.^2)/n_save)
Err_H1_avg=sqrt(sum(Err_H1.^2)/n_save)


%%
[XX,TT]=meshgrid(x, domain.range(1):domain.tsave:domain.range(2));
figure
mesh(TT',XX', u_rom);
colormap(jet)
xlabel('x'); ylabel('t');
xlim([0, 1]);
ylim([0, 1]);
zlim([-0.2, 1.5]);




