%--------------------------------------------------------------------------
% 2S-DD-VMS-ROM Code for 1D Burger's equation
% C. Mou 02/16/2020
%--------------------------------------------------------------------------
clc; clear; close all

addpath ../ROM_FUN/
%addpath ../../FEM/FEM_FUN/
addpath ../DD_FUN/

format long e

global W_w_g W_phi W_p_x
global t_initial t_final
global n_nodes n_gauss
global domain
%------------------------------------------
%--- specify test problem
opt.es     = 0;          % choose test problem %%just for numbering of the case
opt.rhs    = 0;          % choose equations & rhs
opt.bc     = 0;          % choose boundary conditions
opt.ic     = 3;          % choose initial conditions
opt.theta  = 1;          % choose theta value, if negative, semi-implicit
domain.interval = [0, 1];% spatial interval
elem_order = 2;          % choose the order of element
domain.h   = 1/2^9;      % choose mesh size on spatial space
domain.range = [0, 1];   % time range
domain.t   = 1e-4;       % choose time step on tempral space
domain.tsave = domain.t;
domain.calerr_skip = 1;  % skip some steps when calculate errors

opt.parm_e = 1e-3; % viscosity

%------------------------------------------
Folder_read = '../../FEM_Data/';
[File_shape,File_smm,File_dns] = burgers1d_fom_dir_gen(Folder_read,opt,domain,elem_order);

load(File_dns)
load(File_smm)
load(File_shape)

%------------------------------------------
%---- generate POD basis functions
Ms = Mass; % L^2 space
% Ms = Mass+S; % H^1 space

[pod_all,diag_S,d,eig_cum,eigen_cum_ratio] = pod_basis_gen_fun...
    (u_save,Ms);


r = 10;
d = size(pod_all,2);
% fem.p = x';
% fem.t = e_conn';

pod_u =pod_all(:,1:r);

tic

[Tensor_full,~] = pod1d_trilinear_fun(pod_all,d,x, e_conn,n_gauss);
%[~,Tensor_full,~]=POD_tensor_assemble_1D(d,pod_all,n_gauss,x,e_conn);
toc
Tensor_rom = Tensor_full(1:r,1:r,1:r);

%-----------------
t_initial= 0; t_final = 1;
n_save     = fix((t_final-t_initial)/domain.tsave);


u0 = u_save(:,1); % choose the initial condition; %% now it's the first snapshot
%-----------------
% Mr=pod_u'*Mass*pod_u; %------- Mass matrix in ROM
% Sr=pod_u'*S*pod_u;  %------- Stiffness matrix in ROM
%u_proj_r = Mr\pod_u'*Mass*u_save; % ------- ROM projection of the snapshots

Md = pod_all'*Mass*pod_all; %------- Mass matrix in d dimensional space
Sd = pod_all'*S*pod_all; %------- Stiffness matrix in d dimensional space

Mr = Md(1:r,1:r);
Sr = Sd(1:r,1:r);


u_proj_d = Md\pod_all'*Mass*u_save;
u_proj_r = u_proj_d(1:r,:);
u_r0=Mr\pod_u'*Mass*u0; % ------- ROM projection of the initial condition

% assembling the LHS and RHS of the optimization problem
[rhs_mat,lhs_mat] = burgers1d_two_scale_quadratic_ansatz_gen(Tensor_rom,Tensor_full,u_proj_r,u_proj_d);

[U_tsvd,S_tsvd,V_tsvd] =svd(lhs_mat);
rank_coeff= rank(S_tsvd); % loop times

eig_tsvd = diag(S_tsvd);
total = sum(eig_tsvd);

errors_l2_trial = zeros(rank_coeff,1);

%%
save_index = 1;
f0 = zeros(r,1);
f1 = zeros(r,1);

%----- find the optimal tilde{A} and tilde{B}
for jj =1:rank_coeff
    
    [TildeA,TildeB] =burgers1d_dtsvd_invp_solver(U_tsvd,S_tsvd,V_tsvd,rhs_mat,r,jj);

    [u_r_trial] =burgers1d_closure_rom_newton_online_solver(opt, domain, Mr, Sr, ...
        TildeA,TildeB,...
        u_r0, f0, f1, Tensor_rom,n_save,save_index);
    
    err_temp = sqrt(sum(diag((u_r_trial-u_proj_r)'*Mr*(u_r_trial-u_proj_r)) )/size(u_r_trial,2));
    
    errors_l2_trial(jj) = err_temp;
    
end

%---- choose the best tilde{A} and tilde{B}

min_err_jj = find(errors_l2_trial==min(errors_l2_trial));

[TildeA,TildeB] =burgers1d_dtsvd_invp_solver(U_tsvd,S_tsvd,V_tsvd,rhs_mat,r,min_err_jj);

[u_r] =burgers1d_closure_rom_newton_online_solver(opt, domain, Mr, Sr, ...
    TildeA,TildeB,...
    u_r0, f0, f1, Tensor_rom,n_save,save_index);
    
%% 
%----- project ROM solution to FOM space
u_rom=pod_u*u_r;
u_fom = pod_u*u_proj_r;

%-----Caculate the error btw  FEM and POD ROM
Err_L2=zeros(n_save,1); % L2 error at each time step
Err_H1 = Err_L2;
Err_max = Err_L2; Err_H1_semi = Err_L2;

for i=1:n_save
    [Err_max(i),Err_L2(i),Err_H1_semi(i),Err_H1(i)]=...
        pod1d_error_fun(Mass,S,u_fom(:,i),u_rom(:,i));
end

Err_L2_avg=sqrt(sum(Err_L2.^2)/n_save);
Err_H1_avg=sqrt(sum(Err_H1.^2)/n_save);

%%
[XX,TT]=meshgrid(x, domain.range(1):domain.tsave:domain.range(2));

figure
mesh(TT',XX', u_rom);
colormap(jet)
xlabel('x'); ylabel('t');
xlim([0, 1]);
ylim([0, 1]);
zlim([-0.2, 1.5]);





