
function X = runROM_1traj(X0,dt,tN,romInfo,SigmaT)
% run the ROM closure model. 
%{
There can be different closure terms. 
         dX = AX + X'*B*X + closure terms      dX = X(t+1) - X(t) / dt
Here X is the trajectory of X = (a_i = <u,phi_i>).   
           <u_t,phi_k> = nu * <u_xx,phi_k> - <uu_x,phi_k> + closure terms
==>>        rmMass*dX  = A*X + X'*B*X + F(X) 
==>>         A = -<phi_i', phi_j'>;  B = 0.5*<phi_i*phi_j, phi_k'>

1. Euler-Maruyama ROM: using only the linear and quadratic terms 
         dX = AX + X'*B*X + Aest*X +  X'*B_est*X + noise     dX = X(t+1) - X(t) / dt
2. NAR with bondled other terms. 
%}
nx    = length(X0);    
X     = zeros(nx,tN+1); X(:,1)= X0;
noise = zeros(nx,tN);
if exist('SigmaT','var')
    noise = randn(nx,tN); noise = SigmaT*noise;
end

A = romInfo.A; B= romInfo.B; 
switch romInfo.anastz_type
    %% 1&3: EM-ROM:       dX = X(t+1) - X(t) = dt*((A+A_est)*X + X'*(B+B_est)*X) + noise,
    case {'Afull_Bsym','Adiag_Bsym','Galerkin'}
        Arom  = A+ romInfo.A_est; 
        % if printON==1; fprintf('\n EigenArom: \n'); disp(eig(Arom)); end
        Brom  = B+ romInfo.B_est;
        for t=1:tN
            fxB =zeros(nx,1);
            for i=1:nx; fxB(i) = X0'*Brom(:,:,i)*X0;   end
            X1       = X0+dt*(Arom*X0 + fxB) + noise(:,t);   % EM-ROM: honest ROM
            %{
            Frk4     = rk4_ROM_AB(X0,Arom,Brom,dt);          % EM-infer, RK4 forward, ` ROM
            X1       = X0+dt*Frk4 + noise(:,t);
            %}
            X(:,t+1) = X1;     X0       = X1;
        end
    %% 2. rk4+AB:       dX = X(t+1) - X(t) = c1*RK4(dt,A,B)*dt +dt*(A_est*X + X'*B_est*X) + noise,   
    case {'rk4+AB'}
        c_est =  romInfo.c_est; A_est = romInfo.A_est; B_est = romInfo.B_est;        
        ABx = @(X) A_est*X + computeBterm(B_est,X);
        for t=1:tN
            Frk4     = rk4_ROM_AB(X0,A,B,dt); 
            X1       = X0+dt*(c_est(1:nx).*Frk4 + c_est(nx+1:end)) +dt*ABx(X0)+ noise(:,t);
            X(:,t+1) = X1;     X0       = X1;
        end     
        %% 4. NAR
    case {'NAR'}
        c_est  = romInfo.c_est; A_est = romInfo.A_est; B_est = romInfo.B_est;
        ASL    = romInfo.ASL; BSL = romInfo.BSL;   B2rL = romInfo.B2rL; 
        for t=1:tN
            % Frk4     = rk4_ROM_AB(X0,A,B,dt);
            term1 = rk4_ROM_AB(X0,A,B,dt);
            termQ = NAR_termQ(X0,ASL,BSL,B2rL);
            term_sum = c_est(1:nx).*term1 + c_est(nx+1:2*nx).*termQ + c_est(2*nx+1:end);
            X1       = X0+dt*term_sum +dt*A_est*X0+ noise(:,t);
            X(:,t+1) = X1;     X0       = X1;
        end
end


end