function [x_reg,lambda_opt] = reg_Lcurve_Abnorm_B(A,b,B,curvatureType,plotON)
% regularizaiton using L-curve to select optimal lambda 
%{
                  x_l = argmin_x  |Ax -b|^2 + lambda x'*B*x
                      = (A'*A+lambda*B)\(A'*b)
 Set lambda to damp the error in A\b while keeping the norm x'*B*x small 
 Select lambda that at the "corner" of the L-curve (maximum curvature)
               ( |Ax_l -b|^2, x_l'*B*x_l)   = (xlambda,ylambda)
 See Hansen: the L-curve and its use in the numerical treatment of inverse problems 
 - difference1: we use   lambda x'*B*x instead of  lambda^2 |Lx-x0|^2
 - difference2: we compute curvature using either formula for (x(t),y(t)) or the 3 point circle 
% For our case: | M1X*c - M1Y|^2, so A= M1X, b= M1Y;  
% Copyright(c): Fei Lu, Qingci An, Quanjun Lang 
%}

A1       = A'*A; 
[Q,S,V]  = svd(A1); S  = diag(S);

Eigens      = S(S>1e-10);
minEigen    = max( min(Eigens), 1e-10); maxEigen = min(max(Eigens),0.5);
temp        = linspace(log10(minEigen), log10(maxEigen),100);
lambda_seq  = 10.^temp;
len         = numel(lambda_seq);

res_seq     = zeros(len,1); reg_seq = res_seq;   
bvec        = A'*b; 
for ll = 1:len
    lambda1 = lambda_seq(ll); 
    x_l     = lsqminnorm(A'*A+lambda1*B,bvec); 
    res_seq(ll) = norm(A*x_l - b)^2;
    reg_seq(ll) = x_l'*B*x_l; 
end
    
if ~exist('curvatureType','var');  curvatureType = '3ptCircle'; end
switch curvatureType
    case '3ptCircle'      % use 3point circle radius 
         lambda_opt = Opt_lambda_curvature3pt(res_seq,reg_seq,lambda_seq,plotON); 
    otherwise 
         lambda_opt = opt_lambda_curvature(res_seq,reg_seq, lambda_seq,plotON);
end

x_reg =lsqminnorm(A'*A+lambda_opt*B, bvec); %  (A'*A+lambda_opt*B)\bvec; 
end


