%% get prediction data, takes about 12 minutes to run on a 4 core computer, data .5G 

function [a_Mtraj20,u_r_err20,u_Mtraj] = get_predictData(fem_settings, filenames,pod_all,tN_pred,M_pred)


%% 1. generate Data: num_test new trajectories
saveON     = 1;    % postprocess data
num_test   = 200;  % number of trajectories (with different ICs)   
if exist('M_pred','var') && M_pred>num_test;  num_test = M_pred; end               % pre-compuate these data and load them directly. 

% initialize the FEM solver with pre-determined ICs
ic_modes     = 50; % modes of sine functions that the IC contains
u0_mtraj     = get_IC(filenames,fem_settings,ic_modes,num_test,saveON); % get multiple trajectories's ICs
datapath    =  filenames.Folder_save;
File_snapshots_test = [datapath,sprintf('prediction_Mpred%i_tPred%i',num_test,tN_pred),'.mat'];

if exist(File_snapshots_test,'file')  % generate data if do not exist
    fprintf('Getting prediction data trajectories 1--%i \n',num_test);
    load(File_snapshots_test,'a_Mtraj20','u_Mtraj','u_r_err20');
else
    save_1Traj = 0;
    Nx           = length(u0_mtraj(:,1));  
    tN = tN_pred;   fem_settings.tN =tN;
    fem_settings.t_final = fem_settings.t_initial + tN*fem_settings.domain.t; 
    Kmat_Mtraj   = zeros(Nx,Nx,num_test);
    u_Mtraj      = zeros(Nx,tN+1,num_test);  % if Nx*tN is too large, we do not save it, save ICs instead
    tic
    ticBytes(gcp); % tocBytes Read how many bytes have been transferred since calling ticBytes
    opt = fem_settings.opt; domain = fem_settings.domain; 
    parfor k =1:num_test
        u0 = u0_mtraj(:,k);
        [k_mat,u_save,~,~,~] = burgers1d_fem_1traj_v2(opt,domain,fem_settings,u0,filenames,save_1Traj,k);
        Kmat_Mtraj(:,:,k) = k_mat;
        u_Mtraj(:,:,k)    = u_save;
    end
    tocBytes(gcp);     toc
    % 1.2 get coeffecient a(t) of POD_Basis  
    x_mesh    = fem_settings.fem_mesh.x;  dx = x_mesh(2)-x_mesh(1); 
    pod_all1  = pod_all; %/sqrt(dx);     % the pod mode in mesh should be scaled by /sqrt(dx) to get them to physical space
    a_Mtraj20   = get_at_data(u_Mtraj,pod_all1,20)*dx;
    a_t_all     = get_at_data(u_Mtraj,pod_all1,Nx)*dx;               % get a(t) trajectory data: 20 of them
    u_r_err20   = vecnorm(a_t_all(21:end,:,:)) ;   % the error of the unresolved modes from 21:end; for r-mode error, add the r+1:20 error
%    a_Mtraj20   = a_t_all(1:20,:,:); 
    save(File_snapshots_test,'fem_settings','u_Mtraj','Kmat_Mtraj','dx','a_Mtraj20','u_r_err20');
end






