%% test a sequence of r and timegap, and select the optimal space-time reduction

function [pred_err,errMtraj] = space_time_reduction(a_Mtraj,dt,anastz_type,pod_all1,viscosity,x_mesh,filenames,fem_settings)
r_seq    = [6,8,10,12,14,16];  
gapSeq   = 1:1:15; 
n_rseq   = length(r_seq);
n_tseq   = length(gapSeq);
M_pred   = 20; 

tN_pred  = 800;  tpred = 1:401; 
File_predErr  = [filenames.Folder_save,sprintf('spaceTime_predM%i_',M_pred),anastz_type,'SigmaOFF.mat'];

if ~exist(File_predErr,'file')
    [a_Mtraj_new,u_r_err20,~] = get_predictData(fem_settings, filenames,pod_all1,tN_pred,M_pred);    % get 200 new trajectories, each have 20 modes
    
    pred_err   = zeros(n_tseq,n_rseq);              % mean of Mpred average-trajectory errors
    errMtraj   = zeros(n_tseq,M_pred,n_rseq);       % Mpred average-trajectory errors
    u_errMtraj = errMtraj;                        % solution field error: ||u(x,t) - u_r(x,t)||_L^2
    for i=1:n_rseq
        r = r_seq(i);
        % get estimators for gapSeq
        [~,romInfo_seq,gapSeq]     = getEst_timegapSeq(gapSeq,a_Mtraj(1:r,:,:),dt,anastz_type,pod_all1,viscosity,r,x_mesh,filenames);
        
        % compute error of estimator
         u_r_err = sum(a_Mtraj_new((r+1):end,:,:).^2,1) + u_r_err20.^2; u_r_err = squeeze(u_r_err);
        [pred_err(i,:), err_temp,u_r_err]  = getPredict_error(a_Mtraj_new(1:r,tpred,1:M_pred),romInfo_seq,gapSeq,u_r_err(tpred,1:M_pred));
        u_errMtraj(:,:,i) = sqrt(err_temp.^2+ u_r_err); 
        errMtraj(:,:,i)   = err_temp;
    end
    save(File_predErr,'pred_err','errMtraj','u_errMtraj');
else
    load(File_predErr);
end

%% plot the r-gap imagesc to show space-time reduction 
ind           = isnan(pred_err); pred_err1 = pred_err;
pred_err1(ind)= min(min(pred_err))*0.8;  
figure; 
imagesc(gapSeq,r_seq,pred_err1); xlabel('Time Gap'); ylabel('r');   colorbar;  colormap jet;
figpath = [filenames.Folder_save,'Figures/']; 
figname = 'spaceTime_r_gap';
set_positionFontsAll_43
% print([figpath,figname,'.eps'],'-depsc'); 
print([figpath,figname,'.pdf'],'-dpdf');  % 



end